@@ -44,7 +44,12 @@ func (b *Builder) BuildFunc() interface{} {
 	return b.Build
 }
 
-// BuilderConfig is the configuration structure for the registry.
+// BuildFunc implements component.BuilderODR
+func (b *Builder) BuildODRFunc() interface{} {
+	return b.BuildODR
+}
+
+// BuilderConfig is the configuration structure for the builder
 type BuilderConfig struct {
 	// Control whether or not to inject the entrypoint binary into the resulting image
 	DisableCEB bool `hcl:"disable_entrypoint,optional"`
@@ -63,6 +68,9 @@ type BuilderConfig struct {
 
 	// Controls the passing of build context
 	Context string `hcl:"context,optional"`
+
+	// Controls the passing of the target stage
+	Target string `hcl:"target,optional"`
 }
 
 func (b *Builder) Documentation() (*docs.Documentation, error) {
@@ -154,6 +162,14 @@ build {
 		"Build context path",
 	)
 
+	doc.SetField(
+		"target",
+		"the target build stage in a multi-stage Dockerfile",
+		docs.Summary(
+			"If buildkit is enabled unused stages will be skipped",
+		),
+	)
+
 	return doc, nil
 }
 
@@ -162,6 +178,68 @@ func (b *Builder) Config() (interface{}, error) {
 	return &b.config, nil
 }
 
+// Build
+func (b *Builder) BuildODR(
+	ctx context.Context,
+	ui terminal.UI,
+	src *component.Source,
+	log hclog.Logger,
+	ai *AccessInfo,
+) (*Image, error) {
+	sg := ui.StepGroup()
+	defer sg.Wait()
+
+	dockerfile := b.config.Dockerfile
+	if dockerfile == "" {
+		dockerfile = "Dockerfile"
+	}
+	if !filepath.IsAbs(dockerfile) {
+		dockerfile = filepath.Join(src.Path, dockerfile)
+	}
+
+	// If the dockerfile is outside of our build context, then we copy it
+	// into our build context.
+	relDockerfile, err := filepath.Rel(src.Path, dockerfile)
+	if err != nil || strings.HasPrefix(relDockerfile, "..") {
+		id, err := ulid.New(ulid.Now(), rand.Reader)
+		if err != nil {
+			return nil, err
+		}
+
+		newPath := filepath.Join(src.Path, fmt.Sprintf("Dockerfile-%s", id.String()))
+		if err := copyFile(dockerfile, newPath); err != nil {
+			return nil, err
+		}
+		defer os.Remove(newPath)
+
+		dockerfile = newPath
+	}
+
+	path := src.Path
+
+	if b.config.Context != "" {
+		path = b.config.Context
+	}
+
+	contextDir, relDockerfile, err := build.GetContextFromLocalDir(path, dockerfile)
+	if err != nil {
+		return nil, status.Errorf(codes.FailedPrecondition, "unable to create Docker context: %s", err)
+	}
+	log.Debug("loaded Docker context",
+		"context_dir", contextDir,
+		"dockerfile", relDockerfile,
+	)
+
+	log.Info("executing build via kaniko")
+
+	result, err := b.buildWithKaniko(ctx, ui, sg, log, relDockerfile, contextDir, b.config.BuildArgs, ai)
+	if err != nil {
+		return nil, err
+	}
+
+	return result, nil
+}
+
 // Build
 func (b *Builder) Build(
 	ctx context.Context,
@@ -246,10 +324,7 @@ func (b *Builder) Build(
 		step.Update("Docker isn't available. Falling back to daemonless image build...")
 		step.Done()
 		step = nil
-		if err := b.buildWithImg(
-			ctx, ui, sg, relDockerfile, contextDir, result.Name(),
-			b.config.BuildArgs,
-		); err != nil {
+		if err := b.buildWithImg(ctx, ui, sg, relDockerfile, contextDir, result.Name(), b.config.BuildArgs, b.config.Target); err != nil {
 			return nil, err
 		}
 
@@ -263,9 +338,7 @@ func (b *Builder) Build(
 		// No fallback, build with Docker
 		step.Done()
 		step = nil
-		if err := b.buildWithDocker(
-			ctx, ui, sg, cli, contextDir, relDockerfile, result.Name(), b.config.Platform, b.config.BuildArgs, log,
-		); err != nil {
+		if err := b.buildWithDocker(ctx, ui, sg, cli, contextDir, relDockerfile, result.Name(), b.config.Platform, b.config.BuildArgs, b.config.Target, log); err != nil {
 			return nil, err
 		}
 	}
@@ -322,6 +395,7 @@ func (b *Builder) buildWithDocker(
 	tag string,
 	platform string,
 	buildArgs map[string]*string,
+	target string,
 	log hclog.Logger,
 ) error {
 	excludes, err := build.ReadDockerignore(contextDir)
@@ -341,7 +415,6 @@ func (b *Builder) buildWithDocker(
 		ExcludePatterns: excludes,
 		ChownOpts:       &idtools.Identity{UID: 0, GID: 0},
 	})
-
 	if err != nil {
 		return status.Errorf(codes.Internal, "unable to compress context: %s", err)
 	}
@@ -370,6 +443,7 @@ func (b *Builder) buildWithDocker(
 		Remove:     true,
 		Platform:   platform,
 		BuildArgs:  buildArgs,
+		Target:     target,
 	}
 
 	// Buildkit builds need a session under most circumstances, but sessions are only supported in >1.39
