@@ -126,8 +126,13 @@ type baseCommand struct {
 }
 
 // Close cleans up any resources that the command created. This should be
-// defered by any CLI command that embeds baseCommand in the Run command.
+// deferred by any CLI command that embeds baseCommand in the Run command.
 func (c *baseCommand) Close() error {
+	// Close the project client, which gracefully shuts down the local runner
+	if c.project != nil {
+		c.project.Close()
+	}
+
 	// Close our UI if it implements it. The glint-based UI does for example
 	// to finish up all the CLI output.
 	if closer, ok := c.ui.(io.Closer); ok && closer != nil {
@@ -235,6 +240,7 @@ func (c *baseCommand) Init(opts ...Option) error {
 			match := reAppTarget.FindStringSubmatch(c.args[0])
 			if match != nil {
 				// Set our refs
+				c.refProject = &pb.Ref_Project{Project: match[1]}
 				c.refApp = &pb.Ref_Application{
 					Project:     match[1],
 					Application: match[2],
@@ -309,18 +315,16 @@ func (c *baseCommand) Init(opts ...Option) error {
 
 		c.cfg = cfg
 		if cfg != nil {
+			project := &pb.Ref_Project{Project: cfg.Project}
 
-			// If we require a project target and we still haven't set it,
-			// and the user provided it via the CLI, set it now.
+			// If we're loading config, we'll have a project, and we set it now.
 			// If they didn't provide a value via flag, we default to
 			// the project from initConfig.
-			if (baseCfg.AppOptional || baseCfg.ProjectTargetRequired) &&
-				c.refProject == nil {
-				if c.flagProject != "" {
-					c.refProject = &pb.Ref_Project{Project: c.flagProject}
-				} else {
-					c.refProject = &pb.Ref_Project{Project: cfg.Project}
-				}
+			if c.flagProject != "" {
+				project = &pb.Ref_Project{Project: c.flagProject}
+			}
+			if c.refProject == nil {
+				c.refProject = project
 			}
 
 			// If we require an app target and we still haven't set it,
@@ -331,7 +335,7 @@ func (c *baseCommand) Init(opts ...Option) error {
 				c.refApp == nil &&
 				c.flagApp != "" {
 				c.refApp = &pb.Ref_Application{
-					Project:     c.refProject.Project,
+					Project:     project.Project,
 					Application: c.flagApp,
 				}
 			}
@@ -390,19 +394,6 @@ func (c *baseCommand) Init(opts ...Option) error {
 		}
 	}
 
-	if baseCfg.ProjectTargetRequired {
-		if c.refProject == nil {
-			if len(c.cfg.Project) == 0 {
-				c.ui.Output(errProjectMode, terminal.WithErrorStyle())
-				return ErrSentinel
-			}
-
-			c.refProject = &pb.Ref_Project{
-				Project: c.cfg.Project,
-			}
-		}
-	}
-
 	return nil
 }
 
@@ -756,9 +747,6 @@ This command requires a single targeted app. You have multiple apps defined
 so you can specify the app to target using the "-app" flag.
 `)
 
-	errProjectMode = strings.TrimSpace(`
-This command requires a targeted project.`)
-
 	// matches either "project" or "project/app"
 	reAppTarget = regexp.MustCompile(`^(?P<project>[-0-9A-Za-z_]+)/(?P<app>[-0-9A-Za-z_]+)$`)
 
