@@ -17,16 +17,20 @@ import (
 	"strings"
 	"time"
 
+	"contrib.go.opencensus.io/exporter/ocagent"
+	datadog "github.com/DataDog/opencensus-go-exporter-datadog"
 	"github.com/hashicorp/go-hclog"
 	hznhub "github.com/hashicorp/horizon/pkg/hub"
 	hzntest "github.com/hashicorp/horizon/pkg/testutils/central"
 	wphzn "github.com/hashicorp/waypoint-hzn/pkg/server"
+	"github.com/hashicorp/waypoint/internal/telemetry"
 	"github.com/mitchellh/go-testing-interface"
 	"github.com/posener/complete"
 	"github.com/stretchr/testify/require"
 	bolt "go.etcd.io/bbolt"
 
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
+	"github.com/hashicorp/waypoint/internal/pkg/cert"
 	"github.com/hashicorp/waypoint/internal/pkg/flag"
 	"github.com/hashicorp/waypoint/internal/server"
 	"github.com/hashicorp/waypoint/internal/server/singleprocess"
@@ -68,6 +72,12 @@ type ServerRunCommand struct {
 	flagAcceptTOS              bool
 	flagTLSCertFile            string
 	flagTLSKeyFile             string
+
+	flagTelemetryOpenCensusAgentAddr     string
+	flagTelemetryOpenCensusAgentInsecure bool
+
+	flagTelemetryDatadogTraceAddr     string
+	flagTelemetryOpenCensusZpagesAddr string
 }
 
 func (c *ServerRunCommand) Run(args []string) int {
@@ -190,12 +200,36 @@ func (c *ServerRunCommand) Run(args []string) int {
 	}
 	defer httpLn.Close()
 
+	var httpInsecureLn net.Listener
+	if c.config.HTTPInsecure.Addr != "" {
+		c.config.HTTPInsecure.TLSDisable = true
+		httpInsecureLn, err = c.listenerForConfig(log.Named("http_insecure"), &c.config.HTTPInsecure)
+		if err != nil {
+			c.ui.Output(
+				"Error starting insecure HTTP listener: %s", err.Error(),
+				terminal.WithErrorStyle(),
+			)
+			return 1
+		}
+		defer httpInsecureLn.Close()
+	}
+
+	telemetryEnabled := false
+	// If we have any export address configured, we should enable telemetry.
+	if c.flagTelemetryOpenCensusAgentAddr != "" || c.flagTelemetryDatadogTraceAddr != "" {
+		telemetryEnabled = true
+	}
+
 	options := []server.Option{
 		server.WithContext(c.Ctx),
 		server.WithLogger(log),
 		server.WithGRPC(ln),
 		server.WithHTTP(httpLn),
 		server.WithImpl(impl),
+		server.WithTelemetry(telemetryEnabled),
+	}
+	if httpInsecureLn != nil {
+		options = append(options, server.WithHTTP(httpInsecureLn))
 	}
 	auth := false
 	if ac, ok := impl.(server.AuthChecker); ok {
@@ -218,6 +252,10 @@ func (c *ServerRunCommand) Run(args []string) int {
 		{Name: "gRPC Address", Value: ln.Addr().String()},
 		{Name: "HTTP Address", Value: httpLn.Addr().String()},
 	}
+	if httpInsecureLn != nil {
+		values = append(values, terminal.NamedValue{
+			Name: "HTTP Address (Insecure)", Value: httpInsecureLn.Addr().String()})
+	}
 	if auth {
 		values = append(values, terminal.NamedValue{Name: "Auth Required", Value: "yes"})
 	}
@@ -264,7 +302,7 @@ This command will bootstrap the server and setup a CLI context.
 		closer.Close()
 	}
 
-	// Set our log output higher if its not already so that it begins showing.
+	// Set our log output higher if it's not already so that it begins showing.
 	if !log.IsInfo() {
 		log.SetLevel(hclog.Info)
 	}
@@ -290,6 +328,60 @@ This command will bootstrap the server and setup a CLI context.
 		}
 	}
 
+	if telemetryEnabled {
+		log.Debug("Telemetry enabled - configuring telemetry options.")
+		telemetryOptions := []telemetry.Option{
+			telemetry.WithLogger(log.Named("telemetry")),
+		}
+
+		// ZPages
+		if c.flagTelemetryOpenCensusZpagesAddr != "" {
+			telemetryOptions = append(telemetryOptions, telemetry.WithZpages(c.flagTelemetryOpenCensusZpagesAddr))
+		}
+
+		// OpenCensus Agent
+		if c.flagTelemetryOpenCensusAgentAddr != "" {
+			ocagentOptions := []ocagent.ExporterOption{
+				ocagent.WithAddress(c.flagTelemetryOpenCensusAgentAddr),
+				ocagent.WithServiceName("waypoint"),
+			}
+			if c.flagTelemetryOpenCensusAgentInsecure {
+				ocagentOptions = append(ocagentOptions, ocagent.WithInsecure())
+			}
+
+			telemetryOptions = append(telemetryOptions, telemetry.WithOpenCensusExporter(ocagentOptions))
+		}
+
+		// Datadog
+		if c.flagTelemetryDatadogTraceAddr != "" {
+			telemetryOptions = append(telemetryOptions, telemetry.WithDatadogExporter(
+				datadog.Options{
+					TraceAddr: c.flagTelemetryDatadogTraceAddr,
+					Service:   "waypoint",
+				},
+			))
+		}
+
+		t, err := telemetry.NewTelemetry(telemetryOptions...)
+		if err != nil {
+			c.ui.Output(
+				"Error setting up telemetry: %s", err.Error(),
+				terminal.WithErrorStyle(),
+			)
+			return 1
+		}
+
+		go func() {
+			// Will gracefully exit when the context passed in closes.
+			err := t.Run(c.Ctx)
+			if err != nil {
+				log.Error("Telemetry runner exited with error", "error", err)
+			} else {
+				log.Debug("Telemetry runner completed.")
+			}
+		}()
+	}
+
 	// Run the server
 	log.Info("starting built-in server", "addr", ln.Addr().String())
 	server.Run(options...)
@@ -324,12 +416,21 @@ func (c *ServerRunCommand) Flags() *flag.Sets {
 			Default: "127.0.0.1:9702",
 		})
 
+		f.StringVar(&flag.StringVar{
+			Name:   "listen-http-insecure",
+			Target: &c.config.HTTPInsecure.Addr,
+			Usage: "Address to bind to for insecure HTTP connections. " +
+				"This will not have TLS enabled. This will redirect users to the TLS " +
+				"port UNLESS there is an X-Forwarded-Proto header. This makes this port " +
+				"suitable for proxy backends.",
+		})
+
 		f.StringVar(&flag.StringVar{
 			Name:   "tls-cert-file",
 			Target: &c.flagTLSCertFile,
 			Usage: "Path to a PEM-encoded certificate file for TLS. If this " +
 				"isn't set, a self-signed certificate will be generated. This file " +
-				"will be read once at startup and will not be monitored for changes.",
+				"will be monitored for changes and will automatically reload on change.",
 			Default: "",
 		})
 
@@ -338,7 +439,8 @@ func (c *ServerRunCommand) Flags() *flag.Sets {
 			Target: &c.flagTLSKeyFile,
 			Usage: "Path to a PEM-encoded private key file for the TLS certificate " +
 				"specified with -tls-cert-file. This is required if -tls-cert-file " +
-				"is set.",
+				"is set. This file will be monitored for changes and will automatically " +
+				"reload on change",
 			Default: "",
 		})
 
@@ -367,21 +469,21 @@ func (c *ServerRunCommand) Flags() *flag.Sets {
 		f.StringVar(&flag.StringVar{
 			Name:    "url-api-addr",
 			Target:  &c.config.URL.APIAddress,
-			Usage:   "Address to Waypoint URL service API",
+			Usage:   "Address to Waypoint URL service API.",
 			Default: "api.waypoint.run:443",
 		})
 
 		f.BoolVar(&flag.BoolVar{
 			Name:    "url-api-insecure",
 			Target:  &c.config.URL.APIInsecure,
-			Usage:   "True if TLS is not enabled for the Waypoint URL service API",
+			Usage:   "True if TLS is not enabled for the Waypoint URL service API.",
 			Default: false,
 		})
 
 		f.StringVar(&flag.StringVar{
 			Name:    "url-control-addr",
 			Target:  &c.config.URL.ControlAddress,
-			Usage:   "Address to Waypoint URL service control API",
+			Usage:   "Address to Waypoint URL service control API.",
 			Default: DefaultURLControlAddress,
 		})
 
@@ -425,6 +527,31 @@ func (c *ServerRunCommand) Flags() *flag.Sets {
 			Usage:   acceptTOSHelp,
 			Default: false,
 		})
+		f.StringVar(&flag.StringVar{
+			Name:   "telemetry-oc-agent-addr",
+			Target: &c.flagTelemetryOpenCensusAgentAddr,
+			Usage: "Address of an OpenCensus agent or collector available to receive OpenCensus formatted " +
+				"telemetry, traces and stats (commonly port 55678). Example: localhost:55678",
+		})
+		f.BoolVar(&flag.BoolVar{
+			Name:    "telemetry-oc-agent-insecure",
+			Target:  &c.flagTelemetryOpenCensusAgentInsecure,
+			Usage:   "Disables client transport security for the OpenCensus agent exporter's gRPC connection.",
+			Default: false,
+		})
+		f.StringVar(&flag.StringVar{
+			Name:   "telemetry-dd-trace-addr",
+			Target: &c.flagTelemetryDatadogTraceAddr,
+			Usage:  "Address of a DataDog agent available to accept traces (commonly port 8126). Example: localhost:8126",
+		})
+		f.StringVar(&flag.StringVar{
+			Name:   "telemetry-oc-zpages-addr",
+			Target: &c.flagTelemetryOpenCensusZpagesAddr,
+			Usage: "If set, Waypoint will run an OpenCensus zPages server at this address. Typically this is " +
+				"set to something like localhost:9999, where trace debug information could be viewed at " +
+				"http://localhost:9999/debug/tracez, and rpc stats at http://localhost:55679/debug/rpcz. " +
+				"More information at https://opencensus.io/zpages/",
+		})
 	})
 }
 
@@ -476,16 +603,17 @@ func (c *ServerRunCommand) listenerForConfig(log hclog.Logger, cfg *serverconfig
 		keyFile = c.flagTLSKeyFile
 	}
 
-	var certPEM, keyPEM []byte
+	var tlsConfig *tls.Config
 	if certFile != "" {
-		certPEM, err = ioutil.ReadFile(certFile)
-		if err != nil {
-			return nil, err
-		}
-		keyPEM, err = ioutil.ReadFile(keyFile)
+		w, err := cert.New(log, certFile, keyFile)
 		if err != nil {
 			return nil, err
 		}
+		tlsConfig = w.TLSConfig()
+
+		// NOTE(mitchellh): technically, the cert watcher wants to be closed
+		// to clean up some goroutines, but our listener lives for the duration
+		// of the process so we don't clean it up.
 
 		log.Info("TLS certs loaded from specified files",
 			"cert", certFile,
@@ -493,7 +621,7 @@ func (c *ServerRunCommand) listenerForConfig(log hclog.Logger, cfg *serverconfig
 	}
 
 	// If we don't have a cert then we self-sign.
-	if certPEM == nil {
+	if tlsConfig == nil {
 		log.Info("TLS cert wasn't specified, a self-signed certificate will be created")
 
 		priv, err := ecdsa.GenerateKey(elliptic.P384(), rand.Reader)
@@ -524,27 +652,30 @@ func (c *ServerRunCommand) listenerForConfig(log hclog.Logger, cfg *serverconfig
 		if err != nil {
 			return nil, err
 		}
-		certPEM = out.Bytes()
+		certPEM := out.Bytes()
 
 		// Write the key
 		out = bytes.Buffer{}
 		if err := pem.Encode(&out, pemBlockForKey(priv)); err != nil {
 			return nil, err
 		}
-		keyPEM = out.Bytes()
-	}
+		keyPEM := out.Bytes()
 
-	// Setup the TLS listener
-	cert, err := tls.X509KeyPair(certPEM, keyPEM)
-	if err != nil {
-		ln.Close()
-		return nil, err
+		// Setup the TLS listener
+		cert, err := tls.X509KeyPair(certPEM, keyPEM)
+		if err != nil {
+			ln.Close()
+			return nil, err
+		}
+
+		// Setup our TLS config
+		tlsConfig = &tls.Config{
+			Certificates: []tls.Certificate{cert},
+		}
 	}
 
 	log.Info("listener is wrapped with TLS")
-	return tls.NewListener(ln, &tls.Config{
-		Certificates: []tls.Certificate{cert},
-	}), nil
+	return tls.NewListener(ln, tlsConfig), nil
 }
 
 func publicKey(priv interface{}) interface{} {
