@@ -2,14 +2,12 @@ package runner
 
 import (
 	"context"
-	"encoding/json"
 	"fmt"
 	"io/ioutil"
-	"net"
 	"os"
 	"path/filepath"
 	"reflect"
-	"strings"
+	"sync"
 
 	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-multierror"
@@ -17,7 +15,6 @@ import (
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
 
-	sdk "github.com/hashicorp/waypoint-plugin-sdk"
 	"github.com/hashicorp/waypoint-plugin-sdk/component"
 	"github.com/hashicorp/waypoint-plugin-sdk/datadir"
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
@@ -37,6 +34,8 @@ func (r *Runner) executeJob(
 	ui terminal.UI,
 	job *pb.Job,
 	wd string,
+	clientMutex *sync.Mutex,
+	client pb.Waypoint_RunnerJobStreamClient,
 ) (*pb.Job_Result, error) {
 	// NOTE(mitchellh; krantzinator): For now, we query the project directly here
 	// since we use it only in case of a missing local waypoint.hcl, and to
@@ -51,8 +50,13 @@ func (r *Runner) executeJob(
 		return nil, err
 	}
 
+	// We're going to slowly accumulate configuration info that we use to
+	// update our job later.
+	var configInfo pb.Job_Config
+
 	// Eventually we'll need to extract the data source. For now we're
 	// just building for local exec so it is the working directory.
+	configInfo.Source = pb.Job_Config_FILE
 	path, err := configpkg.FindPath(wd, "", false)
 	if err != nil {
 		return nil, err
@@ -77,6 +81,8 @@ func (r *Runner) executeJob(
 				return nil, status.Errorf(codes.Internal,
 					"Failed to write waypoint.hcl from server: %s", err)
 			}
+
+			configInfo.Source = pb.Job_Config_SERVER
 		} else {
 			log.Trace("waypoint.hcl not found in server data")
 		}
@@ -99,6 +105,21 @@ func (r *Runner) executeJob(
 		return nil, err
 	}
 
+	// Send our first update so we know where the configuration came from.
+	log.Trace("sending initial config load information back to server")
+	clientMutex.Lock()
+	err = client.Send(&pb.RunnerJobStreamRequest{
+		Event: &pb.RunnerJobStreamRequest_ConfigLoad_{
+			ConfigLoad: &pb.RunnerJobStreamRequest_ConfigLoad{
+				Config: &configInfo,
+			},
+		},
+	})
+	clientMutex.Unlock()
+	if err != nil {
+		return nil, err
+	}
+
 	// If we have a project specified on the job, override the configuration
 	// project with that. This allows the same Waypoint configuration to
 	// be shared by multiple projects, which is very possible in the UI,
@@ -124,6 +145,12 @@ func (r *Runner) executeJob(
 		return nil, err
 	}
 
+	// Load variable values from the environment.
+	envVars, diags := variables.LoadEnvValues(cfg.InputVariables)
+	if diags.HasErrors() {
+		return nil, diags
+	}
+
 	// Here we'll load our values from auto vars files and the server/UI, and
 	// combine them with any values set on the job
 	// The order values are added to our final pbVars slice is the order
@@ -134,6 +161,7 @@ func (r *Runner) executeJob(
 	}
 
 	pbVars := resp.Project.GetVariables()
+	pbVars = append(pbVars, envVars...)
 	pbVars = append(pbVars, vcsVars...)
 	pbVars = append(pbVars, job.Variables...)
 
@@ -228,6 +256,9 @@ func (r *Runner) executeJob(
 	case *pb.Job_StatusReport:
 		return r.executeStatusReportOp(ctx, log, job, project)
 
+	case *pb.Job_Init:
+		return r.executeInitOp(ctx, log, project)
+
 	default:
 		return nil, status.Errorf(codes.Aborted, "unknown operation %T", job.Operation)
 	}
@@ -256,7 +287,7 @@ func (r *Runner) pluginFactories(
 	reattachPluginsStr := os.Getenv("WP_REATTACH_PLUGINS")
 	if reattachPluginsStr != "" {
 		var err error
-		reattachPluginConfigs, err = parseReattachPlugins(reattachPluginsStr)
+		reattachPluginConfigs, err = plugin.ParseReattachPlugins(reattachPluginsStr)
 		if err != nil {
 			return nil, err
 		}
@@ -318,46 +349,6 @@ func (r *Runner) pluginFactories(
 	return result, perr
 }
 
-// parse information on reattaching to plugins out of a
-// JSON-encoded environment variable.
-func parseReattachPlugins(in string) (map[string]*goplugin.ReattachConfig, error) {
-	reattachConfigs := map[string]*goplugin.ReattachConfig{}
-	if in != "" {
-		in = strings.TrimRight(in, "'")
-		in = strings.TrimLeft(in, "'")
-		var m map[string]sdk.ReattachConfig
-		err := json.Unmarshal([]byte(in), &m)
-		if err != nil {
-			return reattachConfigs, fmt.Errorf("Invalid format for WP_REATTACH_PROVIDERS: %w", err)
-		}
-		for p, c := range m {
-			var addr net.Addr
-			switch c.Addr.Network {
-			case "unix":
-				addr, err = net.ResolveUnixAddr("unix", c.Addr.String)
-				if err != nil {
-					return reattachConfigs, fmt.Errorf("Invalid unix socket path %q for %q: %w", c.Addr.String, p, err)
-				}
-			case "tcp":
-				addr, err = net.ResolveTCPAddr("tcp", c.Addr.String)
-				if err != nil {
-					return reattachConfigs, fmt.Errorf("Invalid TCP address %q for %q: %w", c.Addr.String, p, err)
-				}
-			default:
-				return reattachConfigs, fmt.Errorf("Unknown address type %q for %q", c.Addr.String, p)
-			}
-			reattachConfigs[p] = &goplugin.ReattachConfig{
-				Protocol:        goplugin.Protocol(c.Protocol),
-				ProtocolVersion: c.ProtocolVersion,
-				Pid:             c.Pid,
-				Test:            c.Test,
-				Addr:            addr,
-			}
-		}
-	}
-	return reattachConfigs, nil
-}
-
 // operationNoDataFunc is the function type for operations that are
 // executed without data downloaded.
 type operationNoDataFunc func(*Runner, context.Context, hclog.Logger, *pb.Job) (*pb.Job_Result, error)
