@@ -6,6 +6,7 @@ import (
 	"github.com/golang/protobuf/ptypes/empty"
 
 	pb "github.com/hashicorp/waypoint/internal/server/gen"
+	"github.com/hashicorp/waypoint/internal/server/ptypes"
 )
 
 func (s *service) SetConfig(
@@ -23,6 +24,10 @@ func (s *service) GetConfig(
 	ctx context.Context,
 	req *pb.ConfigGetRequest,
 ) (*pb.ConfigGetResponse, error) {
+	if err := ptypes.ValidateGetConfigRequest(req); err != nil {
+		return nil, err
+	}
+
 	vars, err := s.state.ConfigGet(req)
 	if err != nil {
 		return nil, err
@@ -35,6 +40,10 @@ func (s *service) SetConfigSource(
 	ctx context.Context,
 	req *pb.SetConfigSourceRequest,
 ) (*empty.Empty, error) {
+	if err := ptypes.ValidateSetConfigSourceRequest(req); err != nil {
+		return nil, err
+	}
+
 	if err := s.state.ConfigSourceSet(req.ConfigSource); err != nil {
 		return nil, err
 	}
@@ -46,6 +55,10 @@ func (s *service) GetConfigSource(
 	ctx context.Context,
 	req *pb.GetConfigSourceRequest,
 ) (*pb.GetConfigSourceResponse, error) {
+	if err := ptypes.ValidateGetConfigSourceRequest(req); err != nil {
+		return nil, err
+	}
+
 	vars, err := s.state.ConfigSourceGet(req)
 	if err != nil {
 		return nil, err
