@@ -2,6 +2,7 @@ package serverinstall
 
 import (
 	"context"
+	json "encoding/json"
 	"fmt"
 	"strconv"
 	"strings"
@@ -24,13 +25,22 @@ type NomadInstaller struct {
 }
 
 type nomadConfig struct {
-	authSoftFail             bool              `hcl:"auth_soft_fail,optional"`
-	serverImage              string            `hcl:"server_image,optional"`
-	namespace                string            `hcl:"namespace,optional"`
-	serviceAnnotations       map[string]string `hcl:"service_annotations,optional"`
-	consulService            bool              `hcl:"consul_service,optional"`
-	consulServiceUITags      []string          `hcl:"consul_service_ui_tags:optional"`
-	consulServiceBackendTags []string          `hcl:"consul_service_backend_tags:optional"`
+	authSoftFail       bool              `hcl:"auth_soft_fail,optional"`
+	serverImage        string            `hcl:"server_image,optional"`
+	namespace          string            `hcl:"namespace,optional"`
+	serviceAnnotations map[string]string `hcl:"service_annotations,optional"`
+
+	consulService            bool     `hcl:"consul_service,optional"`
+	consulServiceUITags      []string `hcl:"consul_service_ui_tags:optional"`
+	consulServiceBackendTags []string `hcl:"consul_service_backend_tags:optional"`
+	consulDatacenter         string   `hcl:"consul_datacenter,optional"`
+	consulDomain             string   `hcl:"consul_datacenter,optional"`
+
+	// If set along with consul, will use this hostname instead of
+	// making a consul DNS hostname for the server address in its context
+	consulServiceHostname string `hcl:"consul_service_hostname,optional"`
+
+	odrImage string `hcl:"odr_image,optional"`
 
 	region         string   `hcl:"namespace,optional"`
 	datacenters    []string `hcl:"datacenters,optional"`
@@ -46,6 +56,8 @@ type nomadConfig struct {
 	csiVolumeCapacityMin int64  `hcl:"csi_volume_capacity_min,optional"`
 	csiVolumeCapacityMax int64  `hcl:"csi_volume_capacity_max,optional"`
 	csiFS                string `hcl:"csi_fs,optional"`
+
+	nomadHost string `hcl:"nomad_host,optional"`
 }
 
 var (
@@ -59,6 +71,17 @@ var (
 	defaultCSIVolumeCapacityMax = int64(2147483648)
 
 	defaultCSIVolumeMountFS = "xfs"
+
+	// Defaults to use for setting up Consul
+	defaultConsulServiceTag       = "waypoint"
+	defaultConsulDatacenter       = "dc1"
+	defaultConsulDomain           = "consul"
+	waypointConsulBackendName     = "waypoint-server"
+	waypointConsulUIName          = "waypoint-ui"
+	defaultWaypointConsulHostname = fmt.Sprintf("%s.%s.service.%s.%s",
+		defaultConsulServiceTag, waypointConsulBackendName, defaultConsulDatacenter, defaultConsulDomain)
+
+	defaultNomadHost = "http://localhost:4646"
 )
 
 // Install is a method of NomadInstaller and implements the Installer interface to
@@ -100,6 +123,14 @@ func (i *NomadInstaller) Install(
 		}
 	}
 
+	if i.config.odrImage == "" {
+		var err error
+		i.config.odrImage, err = defaultODRImage(i.config.serverImage)
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	var (
 		clicfg   clicontext.Config
 		addr     pb.ServerConfig_AdvertiseAddr
@@ -197,16 +228,47 @@ func (i *NomadInstaller) Install(
 		return nil, err
 	}
 
-	serverAddr, err := getAddrFromAllocID(allocID, client)
-	if err != nil {
-		return nil, err
+	// If a Consul service was requested, set the consul DNS hostname rather
+	// than the direct static IP for the CLI context and server config. Otherwise
+	// if Nomad restarts the server allocation, a new IP will be assigned and any
+	// configured clients will be invalid
+	if i.config.consulService {
+		s.Update("Configuring the server context to use Consul DNS hostname")
+		if i.config.consulDatacenter == "" {
+			i.config.consulDatacenter = defaultConsulDatacenter
+		}
+		if i.config.consulDomain == "" {
+			i.config.consulDomain = defaultConsulDomain
+		}
+
+		grpcPort, _ := strconv.Atoi(defaultGrpcPort)
+		httpPort, _ := strconv.Atoi(defaultHttpPort)
+
+		if i.config.consulServiceHostname == "" {
+			addr.Addr = fmt.Sprintf("%s.service.%s.%s:%d",
+				waypointConsulBackendName, i.config.consulDatacenter, i.config.consulDomain, grpcPort)
+			httpAddr = fmt.Sprintf("%s.service.%s.%s:%d",
+				waypointConsulUIName, i.config.consulDatacenter, i.config.consulDomain, httpPort)
+		} else {
+			addr.Addr = fmt.Sprintf("%s:%d", i.config.consulServiceHostname, grpcPort)
+			httpAddr = fmt.Sprintf("%s:%d", i.config.consulServiceHostname, httpPort)
+		}
+	} else {
+		s.Update("Configuring the server context to use the static IP address from the Nomad allocation")
+
+		serverAddr, err := getAddrFromAllocID(allocID, client)
+		if err != nil {
+			return nil, err
+		}
+		hAddr, err := getHTTPFromAllocID(allocID, client)
+		if err != nil {
+			return nil, err
+		}
+
+		httpAddr = hAddr
+		addr.Addr = serverAddr
 	}
-	hAddr, err := getHTTPFromAllocID(allocID, client)
-	if err != nil {
-		return nil, err
-	}
-	httpAddr = hAddr
-	addr.Addr = serverAddr
+
 	clicfg = clicontext.Config{
 		Server: serverconfig.Client{
 			Address:       addr.Addr,
@@ -219,11 +281,20 @@ func (i *NomadInstaller) Install(
 	s.Update("Waypoint server ready")
 	s.Done()
 
-	ui.Output(
-		"WARNING - the Waypoint server running on Nomad is being accessed via its allocation IP and port.\n"+
-			"This could change in the future if Nomad creates a new allocation for the Waypoint server, \n"+
-			"which would break all existing Waypoint contexts.", terminal.WithWarningStyle(),
-	)
+	if i.config.consulService {
+		s = sg.Add("The CLI has been configured to automatically install a Consul service for\n" +
+			"the Waypoint service backend and ui service in Nomad.")
+		s.Done()
+	} else {
+		s = sg.Add(
+			" Waypoint server running on Nomad is being accessed via its allocation IP and port.\n" +
+				"This could change in the future if Nomad creates a new allocation for the Waypoint server,\n" +
+				"which would break all existing Waypoint contexts.\n\n" +
+				"It is recommended to use Consul for determining Waypoint servers IP running on Nomad rather than\n" +
+				"relying on the static IP that is initially set up for this allocation.")
+		s.Status(terminal.StatusWarn)
+		s.Done()
+	}
 
 	return &InstallResults{
 		Context:       &clicfg,
@@ -243,6 +314,15 @@ func (i *NomadInstaller) Upgrade(
 	sg := ui.StepGroup()
 	defer sg.Wait()
 
+	if !i.config.consulService {
+		// By default, we don't auto-enable the consul service because prior to Waypoint
+		// version 0.6.2, we did not enable it by default.
+		sw := sg.Add("Nomad Consul Service is disabled. If you had previously enabled " +
+			"it in the last installation, please stop this upgrade and re-run with -nomad-consul-service=true.")
+		sw.Status(terminal.StatusWarn)
+		sw.Done()
+	}
+
 	s := sg.Add("Initializing Nomad client...")
 	defer func() { s.Abort() }()
 
@@ -274,6 +354,14 @@ func (i *NomadInstaller) Upgrade(
 		}
 	}
 
+	if i.config.odrImage == "" {
+		var err error
+		i.config.odrImage, err = defaultODRImage(i.config.serverImage)
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	clicfg.Server = serverconfig.Client{
 		Tls:           true,
 		TlsSkipVerify: true,
@@ -350,9 +438,8 @@ EVAL:
 		// evaluations ID from eval, because if the upgrade job is identical to what is
 		// currently running, we won't get back a list of allocations, which will
 		// fail the upgrade with no allocations running
-		s.Update("Getting allocations for nomad server job")
+		s.Update("Getting allocations for nomad server job, this may take a while...")
 		allocs, qmeta, err := client.Jobs().Allocations(serverName, false, qopts)
-		s.Update("Got allocations for server install job")
 
 		if err != nil {
 			return nil, err
@@ -363,6 +450,8 @@ EVAL:
 			return nil, fmt.Errorf("no allocations found after evaluation completed")
 		}
 
+		s.Update("Got allocations for server install job")
+
 		switch allocs[0].ClientStatus {
 		case "running":
 			allocID = allocs[0].ID
@@ -386,21 +475,53 @@ EVAL:
 		}
 	}
 
-	serverAddr, err := getAddrFromAllocID(allocID, client)
-	if err != nil {
-		return nil, err
+	// If a Consul service was requested, set the consul DNS hostname rather
+	// than the direct static IP for the CLI context and server config. Otherwise
+	// if Nomad restarts the server allocation, a new IP will be assigned and any
+	// configured clients will be invalid
+	if i.config.consulService {
+		s.Update("Configuring the server context to use Consul DNS hostname")
+		if i.config.consulDatacenter == "" {
+			i.config.consulDatacenter = defaultConsulDatacenter
+		}
+		if i.config.consulDomain == "" {
+			i.config.consulDomain = defaultConsulDomain
+		}
+
+		grpcPort, _ := strconv.Atoi(defaultGrpcPort)
+		httpPort, _ := strconv.Atoi(defaultHttpPort)
+
+		if i.config.consulServiceHostname == "" {
+			addr.Addr = fmt.Sprintf("%s.service.%s.%s:%d",
+				waypointConsulBackendName, i.config.consulDatacenter, i.config.consulDomain, grpcPort)
+			httpAddr = fmt.Sprintf("%s.service.%s.%s:%d",
+				waypointConsulUIName, i.config.consulDatacenter, i.config.consulDomain, httpPort)
+		} else {
+			addr.Addr = fmt.Sprintf("%s:%d", i.config.consulServiceHostname, grpcPort)
+			httpAddr = fmt.Sprintf("%s:%d", i.config.consulServiceHostname, httpPort)
+		}
+	} else {
+		s.Update("Configuring the server context to use the static IP address from the Nomad allocation")
+
+		serverAddr, err := getAddrFromAllocID(allocID, client)
+		if err != nil {
+			return nil, err
+		}
+		hAddr, err := getHTTPFromAllocID(allocID, client)
+		if err != nil {
+			return nil, err
+		}
+
+		httpAddr = hAddr
+		addr.Addr = serverAddr
 	}
-	hAddr, err := getHTTPFromAllocID(allocID, client)
-	if err != nil {
-		return nil, err
-	}
-	httpAddr = hAddr
-	addr.Addr = serverAddr
+
 	clicfg = clicontext.Config{
 		Server: serverconfig.Client{
 			Address:       addr.Addr,
 			Tls:           true,
 			TlsSkipVerify: true, // always for now
+			Platform:      "nomad",
 		},
 	}
 
@@ -704,17 +825,17 @@ func waypointNomadJob(c nomadConfig, rawRunFlags []string) *api.Job {
 	grpcPort, _ := strconv.Atoi(defaultGrpcPort)
 	httpPort, _ := strconv.Atoi(defaultHttpPort)
 
-	// Include services to be registered in Consul, if specified in the server install
+	// Include services to be registered in Consul. Currently configured to happen by default
 	// One service added for Waypoint UI, and one for Waypoint backend port
 	if c.consulService {
 		tg.Services = []*api.Service{
 			{
-				Name:      "waypoint-ui",
+				Name:      waypointConsulUIName,
 				PortLabel: "ui",
 				Tags:      c.consulServiceUITags,
 			},
 			{
-				Name:      "waypoint-server",
+				Name:      waypointConsulBackendName,
 				PortLabel: "server",
 				Tags:      c.consulServiceBackendTags,
 			},
@@ -724,12 +845,6 @@ func waypointNomadJob(c nomadConfig, rawRunFlags []string) *api.Job {
 	tg.Networks = []*api.NetworkResource{
 		{
 			Mode: "host",
-			DynamicPorts: []api.Port{
-				{
-					Label: "server",
-					To:    grpcPort,
-				},
-			},
 			// currently set to static; when ui command can be dynamic - update this
 			ReservedPorts: []api.Port{
 				{
@@ -737,6 +852,11 @@ func waypointNomadJob(c nomadConfig, rawRunFlags []string) *api.Job {
 					Value: httpPort,
 					To:    httpPort,
 				},
+				{
+					Label: "server",
+					To:    grpcPort,
+					Value: grpcPort,
+				},
 			},
 		},
 	}
@@ -878,6 +998,13 @@ func waypointRunnerNomadJob(c nomadConfig, opts *InstallRunnerOpts) *api.Job {
 		value := line[idx+1:]
 		task.Env[key] = value
 	}
+
+	// Let the runner know about the Nomad IP
+	if c.nomadHost == "" {
+		c.nomadHost = defaultNomadHost
+	}
+	task.Env["NOMAD_ADDR"] = c.nomadHost
+
 	tg.AddTask(task)
 
 	return job
@@ -917,6 +1044,46 @@ func getHTTPFromAllocID(allocID string, client *api.Client) (string, error) {
 	return "", nil
 }
 
+func (i *NomadInstaller) OnDemandRunnerConfig() *pb.OnDemandRunnerConfig {
+	// Generate some configuration
+	cfgMap := map[string]interface{}{}
+	if v := i.config.runnerResourcesCPU; v != "" {
+		cfgMap["resources_cpu"] = v
+	}
+	if v := i.config.runnerResourcesMemory; v != "" {
+		cfgMap["resources_memory"] = v
+	}
+	if v := i.config.datacenters[0]; v != "" {
+		cfgMap["datacenter"] = v
+	}
+	if v := i.config.namespace; v != "" {
+		cfgMap["namespace"] = v
+	}
+	if v := i.config.region; v != "" {
+		cfgMap["region"] = v
+	}
+	if v := i.config.nomadHost; v != "" {
+		cfgMap["nomad_host"] = v
+	}
+
+	// Marshal our config
+	cfgJson, err := json.MarshalIndent(cfgMap, "", "\t")
+	if err != nil {
+		// This shouldn't happen cause we control our input. If it does,
+		// just panic cause this will be in a `server install` CLI and
+		// we want the user to report a bug.
+		panic(err)
+	}
+
+	return &pb.OnDemandRunnerConfig{
+		OciUrl:       i.config.odrImage,
+		PluginType:   "nomad",
+		Default:      true,
+		PluginConfig: cfgJson,
+		ConfigFormat: pb.Project_JSON,
+	}
+}
+
 func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 	set.StringMapVar(&flag.StringMapVar{
 		Name:   "nomad-annotate-service",
@@ -939,6 +1106,13 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 		Usage:   "Datacenters to install to for Nomad.",
 	})
 
+	set.StringVar(&flag.StringVar{
+		Name:    "nomad-host",
+		Target:  &i.config.nomadHost,
+		Default: defaultNomadHost,
+		Usage:   "Hostname of the Nomad server to use, like for launching on-demand tasks.",
+	})
+
 	set.StringVar(&flag.StringVar{
 		Name:    "nomad-namespace",
 		Target:  &i.config.namespace,
@@ -946,6 +1120,13 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 		Usage:   "Namespace to install the Waypoint server into for Nomad.",
 	})
 
+	set.StringVar(&flag.StringVar{
+		Name:   "nomad-odr-image",
+		Target: &i.config.odrImage,
+		Usage: "Docker image for the on-demand runners. If not specified, it " +
+			"defaults to the server image name + '-odr' (i.e. 'hashicorp/waypoint-odr:latest')",
+	})
+
 	set.BoolVar(&flag.BoolVar{
 		Name:    "nomad-policy-override",
 		Target:  &i.config.policyOverride,
@@ -998,20 +1179,45 @@ func (i *NomadInstaller) InstallFlags(set *flag.Set) {
 	set.BoolVar(&flag.BoolVar{
 		Name:    "nomad-consul-service",
 		Target:  &i.config.consulService,
-		Usage:   "Create service for Waypoint UI in Consul.",
-		Default: false,
+		Usage:   "Create service for Waypoint UI and Server in Consul.",
+		Default: true, // default to true for fresh installs
+	})
+
+	set.StringVar(&flag.StringVar{
+		Name:   "nomad-consul-service-hostname",
+		Target: &i.config.consulServiceHostname,
+		Usage: "If set, will use this hostname for Consul DNS rather than the default, " +
+			"i.e. \"waypoint-server.service.consul\".",
+		Default: "",
 	})
 
 	set.StringSliceVar(&flag.StringSliceVar{
-		Name:   "nomad-consul-service-ui-tags",
-		Target: &i.config.consulServiceUITags,
-		Usage:  "Tags for the Waypoint UI service generated in Consul.",
+		Name:    "nomad-consul-service-ui-tags",
+		Target:  &i.config.consulServiceUITags,
+		Usage:   "Tags for the Waypoint UI service generated in Consul.",
+		Default: []string{defaultConsulServiceTag},
 	})
 
 	set.StringSliceVar(&flag.StringSliceVar{
 		Name:   "nomad-consul-service-backend-tags",
 		Target: &i.config.consulServiceBackendTags,
-		Usage:  "Tags for the Waypoint backend service generated in Consul.",
+		Usage: "Tags for the Waypoint backend service generated in Consul. The 'first' tag " +
+			"will be used when crafting the Consul DNS hostname for accessing Waypoint.",
+		Default: []string{defaultConsulServiceTag},
+	})
+
+	set.StringVar(&flag.StringVar{
+		Name:    "nomad-consul-datacenter",
+		Target:  &i.config.consulDatacenter,
+		Usage:   "The datacenter where Consul is located.",
+		Default: defaultConsulDatacenter,
+	})
+
+	set.StringVar(&flag.StringVar{
+		Name:    "nomad-consul-domain",
+		Target:  &i.config.consulDomain,
+		Usage:   "The domain where Consul is located.",
+		Default: defaultConsulDomain,
 	})
 
 	set.StringVar(&flag.StringVar{
@@ -1070,6 +1276,13 @@ func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
 		Usage:   "Datacenters to install to for Nomad.",
 	})
 
+	set.StringVar(&flag.StringVar{
+		Name:    "nomad-host",
+		Target:  &i.config.nomadHost,
+		Default: "http://localhost:4646",
+		Usage:   "Hostname of the Nomad server to use, like for launching on-demand tasks.",
+	})
+
 	set.StringVar(&flag.StringVar{
 		Name:    "nomad-namespace",
 		Target:  &i.config.namespace,
@@ -1077,6 +1290,13 @@ func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
 		Usage:   "Namespace to install the Waypoint server into for Nomad.",
 	})
 
+	set.StringVar(&flag.StringVar{
+		Name:   "nomad-odr-image",
+		Target: &i.config.odrImage,
+		Usage: "Docker image for the on-demand runners. If not specified, it " +
+			"defaults to the server image name + '-odr' (i.e. 'hashicorp/waypoint-odr:latest')",
+	})
+
 	set.BoolVar(&flag.BoolVar{
 		Name:    "nomad-policy-override",
 		Target:  &i.config.policyOverride,
@@ -1131,6 +1351,50 @@ func (i *NomadInstaller) UpgradeFlags(set *flag.Set) {
 		Target: &i.config.hostVolume,
 		Usage:  "Nomad host volume name.",
 	})
+
+	set.BoolVar(&flag.BoolVar{
+		Name:    "nomad-consul-service",
+		Target:  &i.config.consulService,
+		Usage:   "Create service for Waypoint UI and Server in Consul.",
+		Default: false, // default to false, make sure people opt into this for upgrades
+	})
+
+	set.StringVar(&flag.StringVar{
+		Name:   "nomad-consul-service-hostname",
+		Target: &i.config.consulServiceHostname,
+		Usage: "If set, will use this hostname for Consul DNS rather than the default, " +
+			"i.e. \"waypoint-server.service.consul\".",
+		Default: "",
+	})
+
+	set.StringSliceVar(&flag.StringSliceVar{
+		Name:    "nomad-consul-service-ui-tags",
+		Target:  &i.config.consulServiceUITags,
+		Usage:   "Tags for the Waypoint UI service generated in Consul.",
+		Default: []string{defaultConsulServiceTag},
+	})
+
+	set.StringSliceVar(&flag.StringSliceVar{
+		Name:   "nomad-consul-service-backend-tags",
+		Target: &i.config.consulServiceBackendTags,
+		Usage: "Tags for the Waypoint backend service generated in Consul. The 'first' tag " +
+			"will be used when crafting the Consul DNS hostname for accessing Waypoint.",
+		Default: []string{defaultConsulServiceTag},
+	})
+
+	set.StringVar(&flag.StringVar{
+		Name:    "nomad-consul-datacenter",
+		Target:  &i.config.consulDatacenter,
+		Usage:   "The datacenter where Consul is located.",
+		Default: defaultConsulDatacenter,
+	})
+
+	set.StringVar(&flag.StringVar{
+		Name:    "nomad-consul-domain",
+		Target:  &i.config.consulDomain,
+		Usage:   "The domain where Consul is located.",
+		Default: defaultConsulDomain,
+	})
 }
 
 func (i *NomadInstaller) UninstallFlags(set *flag.Set) {
