@@ -5,7 +5,7 @@ import SessionService from 'waypoint/services/session';
 export default class Header extends Component {
   @service session!: SessionService;
 
-  get canLogout() {
+  get canLogout(): boolean {
     return this.session.authConfigured;
   }
 }
