@@ -5,24 +5,19 @@ import { tracked } from '@glimmer/tracking';
 import ApiService from 'waypoint/services/api';
 import FlashMessagesService from 'waypoint/services/pds-flash-messages';
 import {
-  Deployment,
   Ref,
   ExpediteStatusReportRequest,
-  StatusReport,
   GetJobStreamRequest,
   GetJobStreamResponse,
   Job,
 } from 'waypoint-pb';
+import { DeploymentExtended } from 'waypoint/services/api';
 
 interface StatusReportBarArgs {
-  model: Deployment.AsObject & WithStatusReport;
+  model: DeploymentExtended;
   artifactType: string;
 }
 
-interface WithStatusReport {
-  statusReport?: StatusReport.AsObject;
-}
-
 export default class StatusReportBar extends Component<StatusReportBarArgs> {
   @service api!: ApiService;
   @service('pdsFlashMessages') flashMessages!: FlashMessagesService;
