@@ -3,15 +3,14 @@ import { inject as service } from '@ember/service';
 import ApiService from 'waypoint/services/api';
 import { Ref, GetProjectRequest, Project } from 'waypoint-pb';
 import { Breadcrumb } from 'waypoint/services/breadcrumbs';
+import { Params as ProjectRouteParams } from 'waypoint/routes/workspace/projects/project';
 
-interface ProjectModelParams {
-  project_id: string;
-}
+type Model = Project.AsObject;
 
 export default class WorkspaceProjectsProjectSettings extends Route {
   @service api!: ApiService;
 
-  breadcrumbs(model: Project.AsObject): Breadcrumb[] {
+  breadcrumbs(model: Model): Breadcrumb[] {
     if (!model) return [];
     return [
       {
@@ -27,10 +26,10 @@ export default class WorkspaceProjectsProjectSettings extends Route {
     ];
   }
 
-  async model() {
+  async model(): Promise<Model> {
     // Setup the project request
     let ref = new Ref.Project();
-    let params = this.paramsFor('workspace.projects.project') as ProjectModelParams;
+    let params = this.paramsFor('workspace.projects.project') as ProjectRouteParams;
     ref.setProject(params.project_id);
     let req = new GetProjectRequest();
     req.setProject(ref);
@@ -38,6 +37,12 @@ export default class WorkspaceProjectsProjectSettings extends Route {
     let resp = await this.api.client.getProject(req, this.api.WithMeta());
     let project = resp.getProject();
 
-    return project?.toObject();
+    if (!project) {
+      // In reality the API will return an error in this circumstance
+      // but the types don’t tell us that.
+      throw new Error(`Project ${params.project_id} not found`);
+    }
+
+    return project.toObject();
   }
 }
