@@ -1,7 +1,9 @@
 import Service from '@ember/service';
+import { DEBUG } from '@glimmer/env';
 import { WaypointClient } from 'waypoint-client';
 import SessionService from 'waypoint/services/session';
 import { inject as service } from '@ember/service';
+import { buildWaiter } from '@ember/test-waiters';
 import {
   Application,
   Build,
@@ -10,13 +12,7 @@ import {
   Job,
   ListBuildsRequest,
   ListBuildsResponse,
-  ListDeploymentsRequest,
-  ListDeploymentsResponse,
   ListPushedArtifactsRequest,
-  ListReleasesRequest,
-  ListReleasesResponse,
-  ListStatusReportsRequest,
-  ListStatusReportsResponse,
   OperationOrder,
   Project,
   PushedArtifact,
@@ -26,11 +22,17 @@ import {
   StatusReport,
   UpsertProjectRequest,
   Variable,
+  UI,
 } from 'waypoint-pb';
-import { Metadata } from 'grpc-web';
+import { Request, Metadata, UnaryResponse, UnaryInterceptor } from 'grpc-web';
+import { Message } from 'google-protobuf';
 import { Empty } from 'google-protobuf/google/protobuf/empty_pb';
 import config from 'waypoint/config/environment';
 
+// The docs for @ember/test-waiters recommend building waiters in module scope.
+// https://github.com/emberjs/ember-test-waiters#use-buildwaiter-in-module-scope
+const waiter = buildWaiter('waypoint-api-service:waiter');
+
 const protocolVersions = {
   // These map to upstream protocol versions
   'client-api-protocol': '1,1',
@@ -41,11 +43,35 @@ const protocolVersions = {
   'client-version': 'ui-0.0.1',
 };
 
+// This is an adapter type. It exists to let us transition to
+// UI.DeploymentBundle incrementally.
+export type DeploymentExtended = Deployment.AsObject & {
+  statusReport?: StatusReport.AsObject;
+  pushedArtifact?: PushedArtifact.AsObject;
+};
+
+// This is an adapter type. It exists to let us transition to
+// UI.ReleaseBundle incrementally.
+export type ReleaseExtended = Release.AsObject & {
+  statusReport?: StatusReport.AsObject;
+  pushedArtifact?: PushedArtifact.AsObject;
+};
+
 export default class ApiService extends Service {
   @service session!: SessionService;
   // If the the apiAddress is not set, this will use the /grpc prefix on the
   // same host as the UI is being served from
-  client = new WaypointClient(`${config.apiAddress}/grpc`, null, null);
+  client = new WaypointClient(`${config.apiAddress}/grpc`, null, {
+    unaryInterceptors: this.unaryInterceptors(),
+  });
+
+  unaryInterceptors(): UnaryInterceptor<Message, Message>[] {
+    if (DEBUG) {
+      return [new ApiWaiterUnaryInterceptor()];
+    } else {
+      return [];
+    }
+  }
 
   // Merges metadata with required metadata for the request
   WithMeta(meta?: Metadata): Metadata {
@@ -63,8 +89,8 @@ export default class ApiService extends Service {
     }
   }
 
-  async listDeployments(wsRef: Ref.Workspace, appRef: Ref.Application): Promise<Deployment.AsObject[]> {
-    let req = new ListDeploymentsRequest();
+  async listDeployments(wsRef: Ref.Workspace, appRef: Ref.Application): Promise<DeploymentExtended[]> {
+    let req = new UI.ListDeploymentsRequest();
     req.setWorkspace(wsRef);
     req.setApplication(appRef);
 
@@ -72,9 +98,24 @@ export default class ApiService extends Service {
     order.setDesc(true);
     req.setOrder(order);
 
-    let resp: ListDeploymentsResponse = await this.client.listDeployments(req, this.WithMeta());
+    let resp = await this.client.uI_ListDeployments(req, this.WithMeta());
 
-    return resp.getDeploymentsList().map((d) => d.toObject());
+    // The following is “adapter” logic. It reshapes UI.DeploymentBundle to work
+    // with existing app code so that we can incrementally migrate to
+    // UI.DeploymentBundle.
+    return resp
+      .getDeploymentsList()
+      .map((bundle) => bundle.toObject())
+      .filter(has('deployment'))
+      .map((bundle) => ({
+        ...bundle.deployment,
+        preload: {
+          ...bundle.deployment.preload,
+          deployUrl: bundle.deployUrl,
+        },
+        statusReport: bundle.latestStatusReport,
+        pushedArtifact: bundle.artifact,
+      }));
   }
 
   async listBuilds(wsRef: Ref.Workspace, appRef: Ref.Application): Promise<Build.AsObject[]> {
@@ -117,8 +158,8 @@ export default class ApiService extends Service {
     return result;
   }
 
-  async listReleases(wsRef: Ref.Workspace, appRef: Ref.Application): Promise<Release.AsObject[]> {
-    let req = new ListReleasesRequest();
+  async listReleases(wsRef: Ref.Workspace, appRef: Ref.Application): Promise<ReleaseExtended[]> {
+    let req = new UI.ListReleasesRequest();
     req.setWorkspace(wsRef);
     req.setApplication(appRef);
 
@@ -127,23 +168,19 @@ export default class ApiService extends Service {
     order.setDesc(true);
     req.setOrder(order);
 
-    let resp: ListReleasesResponse = await this.client.listReleases(req, this.WithMeta());
+    let resp = await this.client.uI_ListReleases(req, this.WithMeta());
 
-    return resp.getReleasesList().map((d) => d.toObject());
-  }
-
-  async listStatusReports(wsRef: Ref.Workspace, appRef: Ref.Application): Promise<StatusReport.AsObject[]> {
-    let req = new ListStatusReportsRequest();
-    req.setWorkspace(wsRef);
-    req.setApplication(appRef);
-
-    let order = new OperationOrder();
-    order.setDesc(true);
-    req.setOrder(order);
-
-    let resp: ListStatusReportsResponse = await this.client.listStatusReports(req, this.WithMeta());
-
-    return resp.getStatusReportsList().map((d) => d.toObject());
+    // The following is “adapter” logic. It reshapes UI.ReleaseBundle to work
+    // with existing app code so that we can incrementally migrate to
+    // UI.ReleaseBundle.
+    return resp
+      .getReleasesList()
+      .map((bundle) => bundle.toObject())
+      .filter(has('release'))
+      .map((bundle) => ({
+        ...bundle.release,
+        statusReport: bundle.latestStatusReport,
+      }));
   }
 
   async getLatestStatusReport(
@@ -325,3 +362,45 @@ function applicationFromObject(object: Application.AsObject): Application {
 
   return result;
 }
+
+/**
+ * Takes a string k and returns a type guard that ensures that, for a given
+ * object o, o[k] !== undefined;
+ *
+ * Useful when working with Array.filter, for which automatic type narrowing is
+ * not yet supported.
+ *
+ * @example
+ * ```
+ * let a: { name?: string }[] = [{ name: 'Alice' }, {}];
+ * let b: { name: string }[] = a.filter(has('name'));
+ * ```
+ *
+ * @param key - the property we want to ensure is present
+ * @returns type guard
+ */
+function has<T, K extends keyof T>(key: K): (obj: T) => obj is T & Required<Pick<T, K>> {
+  return function (obj): obj is T & Required<Pick<T, K>> {
+    return !!obj[key];
+  };
+}
+
+/**
+ * This class uses grpc-web’s interceptor system to track requests using ember’s
+ * test waiter system. For more info, see:
+ * https://github.com/emberjs/ember-test-waiters
+ */
+class ApiWaiterUnaryInterceptor implements UnaryInterceptor<Message, Message> {
+  async intercept(
+    request: Request<Message, Message>,
+    invoker: (request: Request<Message, Message>) => Promise<UnaryResponse<Message, Message>>
+  ) {
+    let token = waiter.beginAsync();
+
+    try {
+      return await invoker(request);
+    } finally {
+      waiter.endAsync(token);
+    }
+  }
+}
