@@ -1,5 +1,5 @@
 import { VersionInfo, GetVersionInfoResponse } from 'waypoint-pb';
-import { Response } from 'miragejs';
+import { Response, RouteHandler } from 'miragejs';
 
 function createVersionInfo(): VersionInfo {
   let versionInfo = new VersionInfo();
@@ -11,7 +11,7 @@ function createVersionInfo(): VersionInfo {
   return versionInfo;
 }
 
-export function get(): Response {
+export function get(this: RouteHandler): Response {
   let resp = new GetVersionInfoResponse();
   let versionInfo = createVersionInfo();
   resp.setInfo(versionInfo);
